//////////////////////////////////////////////////////////////         
//      $Id: options.cpp,v 1.21 2001/06/11 14:32:33 toivo Exp $ 
//
// Author: Toivo Pedastse
//
#include "../config.h"

#include "kpackage.h"
#include "managementWidget.h"
#include "pkgInterface.h"
#include "options.h"
#include "cache.h"

#include <qvbox.h>

#include <klocale.h>

extern Opts *opts;
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Options::Options(QWidget *parent)
    : KDialogBase(Tabbed, i18n("Options"), Ok | Apply | Cancel, Ok, parent, 0, false){

    DCache = dc = opts->DCache;
    PCache = pc = opts->PCache;

    if (DCache >= Opts::SESSION) {
      cacheObj::clearDCache(); // clear dir caches if needed
    }
    if (PCache >= Opts::SESSION) {
      cacheObj::clearPCache(); // clear package caches if needed
    }



    {

      QVBox *page = addVBoxPage(i18n("Types"));

      framet = new QGroupBox(1,Qt::Horizontal,i18n("Handle package type"), page);

      int i;
      for (i = 0; i < kpinterfaceN; i++)  {
	if (kpinterface[i]) {
	  QGroupBox *box = new QGroupBox(2,Qt::Horizontal,kpinterface[i]->name, framet, "box");
	  packageHandle[i] =  new QCheckBox(i18n("Enable"), box);
	  connect(packageHandle[i], SIGNAL(clicked()), this, SLOT(scanLocates()));
	  locate[i] = new QPushButton(i18n("Location of packages"),box);
	  connect(locate[i], SIGNAL(clicked()), kpinterface[i], SLOT(setLocation()));
	} else {
	  packageHandle[i] =  0;
	}
      }
    }

    {
      QVBox *page = addVBoxPage(i18n("Cache"));
      
      bc = new QButtonGroup(page);
      bc->setTitle(i18n("Cache remote package directories"));
      connect( bc, SIGNAL(clicked(int)), SLOT(PDCache(int)) );

      QVBoxLayout* vc = new QVBoxLayout( bc, 15, 10, "vc");
      vc->addSpacing( bc->fontMetrics().height() );

      dcache[0] = new QRadioButton(i18n("Always"),bc);
      vc->addWidget(dcache[0]);

      dcache[1] = new QRadioButton(i18n("During a session"),bc);
      vc->addWidget(dcache[1]);

      dcache[2] = new QRadioButton(i18n("Never"),bc);
      vc->addWidget(dcache[2]);

      bp = new QButtonGroup(page);
      bp->setTitle(i18n("Cache remote package files"));
      connect( bp, SIGNAL(clicked(int)), SLOT(PPCache(int)) );

      QVBoxLayout* vp = new QVBoxLayout( bp, 15, 10, "vp");
      vp->addSpacing( bp->fontMetrics().height() );

      pcache[0] = new QRadioButton(i18n("Always"),bp);
      vp->addWidget(pcache[0]);

      pcache[1] = new QRadioButton(i18n("During a session"),bp);
      vp->addWidget(pcache[1]);

      pcache[2] = new QRadioButton(i18n("Never"),bp);
      vp->addWidget(pcache[2]);

    }

    {
      QVBox *page = addVBoxPage(i18n("Misc"));

      framem = new QGroupBox("", page);
      QVBoxLayout* vf = new QVBoxLayout(framem,20,20);

      ssh = new QCheckBox(i18n("Use SSH for privileged commands"), framem, "ssh");
      vf->addWidget(ssh,0,AlignLeft);

      valid = new QCheckBox(i18n("Verify file list"), framem, "valid");
      vf->addWidget(valid,0,AlignLeft);

      pkgRead = new QCheckBox(i18n("Read information from all local package files"), framem, "pkgr");
      vf->addWidget(pkgRead,0,AlignLeft);
    }
    connect( this, SIGNAL(applyClicked()), SLOT(apply_slot()) );
    connect( this, SIGNAL(okClicked()), SLOT(apply_slot()) );
    connect( this, SIGNAL(closeClicked()), SLOT(cancel_slot()) );
    connect( this, SIGNAL(cancelClicked()), SLOT(cancel_slot()) );

    setValues();
}

void Options::setValues() {

  DCache = dc = opts->DCache;
  PCache = pc = opts->PCache;

  int i;
  for (i = 0; i < kpinterfaceN; i++)  {
    if (kpinterface[i]) {
      packageHandle[i]->setChecked(opts->handlePackage[i]);
    }
  }
  scanLocates();

  dcache[DCache]->setChecked(TRUE);
  pcache[PCache]->setChecked(TRUE);

  ssh->setChecked(opts->useSSH);
  valid->setChecked(opts->VerifyFL);
  pkgRead->setChecked(opts->PkgRead);
}

Options::~Options()
{
}

void Options::scanLocates() {
  int i;

  for (i = 0; i < kpinterfaceN; i++) {
    if (kpinterface[i]) {
      if (packageHandle[i]->isChecked()) {
	locate[i]->setEnabled(true);
      } else {
	locate[i]->setEnabled(false);
      }
    }
  }
}

void Options::restore() 
{
  opts->readSettings();
  setValues();
  show();
}

void Options::cancel_slot() 
{
  kdDebug() << "Cancel\n";
  opts->readSettings();
}

 void Options::apply_slot() 
{
  kdDebug() << "Apply\n";
  opts->useSSH = ssh->isChecked();
  opts->VerifyFL = valid->isChecked();
  opts->PkgRead = pkgRead->isChecked();

  opts->DCache = dc;
  opts->PCache = pc;

  int i;
  for (i = 0; i < kpinterfaceN; i++)  {
    if (packageHandle[i]) {
      opts->handlePackage[i] = packageHandle[i]->isChecked();
    }
  }

 scanLocates();
  opts->writeSettings();
}

void Options::PDCache(int r)
{
  dc = r;
}

void Options::PPCache(int r)
{
  pc = r;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

Opts::Opts()
{
readSettings();
}

Opts::~Opts()
{
}

void Opts::readSettings()
{
  
  KConfig *config = kapp->config();

  config->setGroup("Kpackage");

  DCache = config->readNumEntry("Dir_Cache",1);
  if (DCache >2) {
    kdDebug() << "DCache=" << DCache << "\n";
    DCache = 1;
  }
  PCache = config->readNumEntry("Package_Cache",0);
  if (PCache >2) {
    kdDebug() << "PCache=" << PCache << "\n";
    PCache = 0;
  }

  useSSH = config->readNumEntry("Use_SSH",0);
  VerifyFL = config->readNumEntry("Verify_File_List",1);
  PkgRead = config->readNumEntry("Read_Package_files",0);

  int i;
  for (i = 0; i < kpinterfaceN; i++)  {
    if (kpinterface[i])
      handlePackage[i] = config->readBoolEntry(kpinterface[i]->head,
						      kpinterface[i]->defaultHandle);
  }
}

void Opts::writeSettings()
{
  
  KConfig *config = kapp->config();

  config->setGroup("Kpackage");

  config->writeEntry("Dir_Cache", DCache);
  config->writeEntry("Package_Cache", PCache);

  config->writeEntry("Use_SSH",useSSH );
  config->writeEntry("Verify_File_List",VerifyFL );
  config->writeEntry("Read_Package_files", PkgRead);

  int i;
  for (i = 0; i < kpinterfaceN; i++)  {
    if (kpinterface[i])
      config->writeEntry(kpinterface[i]->head, handlePackage[i]);
  }
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include "options.moc"
